@@ -12,12 +12,13 @@ if TYPE_CHECKING:
     from ansiblelint.errors import MatchError
     from ansiblelint.file_utils import Lintable
     from ansiblelint.rules import RulesCollection
+    from ansiblelint.utils import Task
 
 _logger = logging.getLogger(__name__)
 LOAD_FAILURE_MD = """\
 # load-failure
 
-Linter failed to process a YAML file, probably because it is either:
+"Linter failed to process a file, possible invalid file. Possible reasons:
 
 * contains unsupported encoding (only UTF-8 is supported)
 * not an Ansible file
@@ -53,7 +54,7 @@ class BaseRule:
     _collection: RulesCollection | None = None
 
     @property
-    def help(self) -> str:
+    def help(self) -> str:  # noqa: A003
         """Return a help markdown string for the rule."""
         if self._help is None:
             self._help = ""
@@ -69,7 +70,7 @@ class BaseRule:
     def url(self) -> str:
         """Return rule documentation url."""
         url = self.link
-        if not url:
+        if not url:  # pragma: no cover
             url = RULE_DOC_URL
             if self.id:
                 url += self.id + "/"
@@ -87,7 +88,7 @@ class BaseRule:
             for method in [self.matchlines, self.matchtasks, self.matchyaml]:
                 try:
                     matches.extend(method(file))
-                except Exception as exc:  # pylint: disable=broad-except
+                except Exception as exc:  # pylint: disable=broad-except # noqa: BLE001
                     _logger.warning(
                         "Ignored exception from %s.%s while processing %s: %s",
                         self.__class__.__name__,
@@ -104,7 +105,9 @@ class BaseRule:
         return []
 
     def matchtask(
-        self, task: dict[str, Any], file: Lintable | None = None
+        self,
+        task: Task,
+        file: Lintable | None = None,
     ) -> bool | str | MatchError | list[MatchError]:
         """Confirm if current rule is matching a specific task.
 
@@ -140,7 +143,7 @@ class BaseRule:
 
     def __lt__(self, other: BaseRule) -> bool:
         """Enable us to sort rules by their id."""
-        return (self._order, self.id) < (other._order, other.id)
+        return (self._order, self.id) < (other._order, other.id)  # noqa: SLF001
 
     def __repr__(self) -> str:
         """Return a AnsibleLintRule instance representation."""
@@ -154,6 +157,7 @@ class RuntimeErrorRule(BaseRule):
     """Unexpected internal error."""
 
     id = "internal-error"
+    shortdesc = "Unexpected internal error"
     severity = "VERY_HIGH"
     tags = ["core"]
     version_added = "v5.0.0"
@@ -175,11 +179,11 @@ class LoadingFailureRule(BaseRule):
     """Failed to load or parse file."""
 
     id = "load-failure"
-    description = "Linter failed to process a YAML file, possible not an Ansible file."
+    description = "Linter failed to process a file, possible invalid file."
     severity = "VERY_HIGH"
     tags = ["core", "unskippable"]
     version_added = "v4.3.0"
-    help = LOAD_FAILURE_MD
+    _help = LOAD_FAILURE_MD
     _order = 0
 
 
