@@ -4,28 +4,25 @@ from __future__ import annotations
 import copy
 import logging
 import os
-import pathlib
 import subprocess
 import sys
-from argparse import Namespace
 from collections import OrderedDict, defaultdict
 from contextlib import contextmanager
 from pathlib import Path
 from tempfile import NamedTemporaryFile
-from typing import TYPE_CHECKING, Any, Iterator, cast
+from typing import TYPE_CHECKING, Any, cast
 
-# import wcmatch
 import wcmatch.pathlib
-from wcmatch.wcmatch import RECURSIVE, WcMatch
+import wcmatch.wcmatch
+from yaml.error import YAMLError
 
-from ansiblelint.config import BASE_KINDS, options
-from ansiblelint.constants import FileType
+from ansiblelint.config import BASE_KINDS, Options, options
+from ansiblelint.constants import CONFIG_FILENAMES, GIT_CMD, FileType, States
+from ansiblelint.logger import warn_or_fail
 
 if TYPE_CHECKING:
-    # https://github.com/PyCQA/pylint/issues/3979
-    BasePathLike = os.PathLike[Any]  # pylint: disable=unsubscriptable-object
-else:
-    BasePathLike = os.PathLike
+    from collections.abc import Iterator, Sequence
+
 
 _logger = logging.getLogger(__package__)
 
@@ -33,10 +30,8 @@ _logger = logging.getLogger(__package__)
 def abspath(path: str, base_dir: str) -> str:
     """Make relative path absolute relative to given directory.
 
-    Args:
-       path (str): the path to make absolute
-       base_dir (str): the directory from which make \
-                       relative paths absolute
+    path (str): the path to make absolute
+    base_dir (str): the directory from which make relative paths absolute.
     """
     if not os.path.isabs(path):
         # Don't use abspath as it assumes path is relative to cwd.
@@ -46,9 +41,8 @@ def abspath(path: str, base_dir: str) -> str:
     return os.path.normpath(path)
 
 
-def normpath(path: str | BasePathLike) -> str:
-    """
-    Normalize a path in order to provide a more consistent output.
+def normpath(path: str | Path) -> str:
+    """Normalize a path in order to provide a more consistent output.
 
     Currently it generates a relative path but in the future we may want to
     make this user configurable.
@@ -81,7 +75,7 @@ def is_relative_to(path: Path, *other: Any) -> bool:
         return False
 
 
-def normpath_path(path: str | BasePathLike) -> Path:
+def normpath_path(path: str | Path) -> Path:
     """Normalize a path in order to provide a more consistent output.
 
     - Any symlinks are resolved.
@@ -107,9 +101,9 @@ def normpath_path(path: str | BasePathLike) -> Path:
 
 
 @contextmanager
-def cwd(path: str | BasePathLike) -> Iterator[None]:
+def cwd(path: Path) -> Iterator[None]:
     """Context manager for temporary changing current working directory."""
-    old_pwd = os.getcwd()
+    old_pwd = Path.cwd()
     os.chdir(path)
     try:
         yield
@@ -132,7 +126,7 @@ def expand_paths_vars(paths: list[str]) -> list[str]:
     return paths
 
 
-def kind_from_path(path: Path, base: bool = False) -> FileType:
+def kind_from_path(path: Path, *, base: bool = False) -> FileType:
     """Determine the file kind based on its name.
 
     When called with base=True, it will return the base file type instead
@@ -152,14 +146,22 @@ def kind_from_path(path: Path, base: bool = False) -> FileType:
                     | wcmatch.pathlib.DOTGLOB
                 ),
             ):
-                return str(k)  # type: ignore
+                return str(k)  # type: ignore[return-value]
 
     if base:
         # Unknown base file type is default
         return ""
 
     if path.is_dir():
-        return "role"
+        known_role_subfolders = ("tasks", "meta", "vars", "defaults", "handlers")
+        for filename in known_role_subfolders:
+            if (path / filename).is_dir():
+                return "role"
+        _logger.debug(
+            "Folder `%s` does not look like a role due to missing any of the common subfolders such: %s.",
+            path,
+            ", ".join(known_role_subfolders),
+        )
 
     if str(path) == "/dev/stdin":
         return "playbook"
@@ -178,18 +180,23 @@ class Lintable:
     When symlinks are given, they will always be resolved to their target.
     """
 
-    def __init__(
+    # pylint: disable=too-many-arguments
+    def __init__(  # noqa: C901
         self,
         name: str | Path,
         content: str | None = None,
         kind: FileType | None = None,
+        base_kind: str = "",
+        parent: Lintable | None = None,
     ):
         """Create a Lintable instance."""
         self.dir: str = ""
         self.kind: FileType | None = None
         self.stop_processing = False  # Set to stop other rules from running
-        self._data: Any = None
+        self.state: Any = States.NOT_LOADED
         self.line_skips: dict[int, set[str]] = defaultdict(set)
+        self.exc: Exception | None = None  # Stores data loading exceptions
+        self.parent = parent
 
         if isinstance(name, str):
             name = Path(name)
@@ -236,16 +243,45 @@ class Lintable:
                 self.dir = str(self.path.parent.resolve())
 
         # determine base file kind (yaml, xml, ini, ...)
-        self.base_kind = kind_from_path(self.path, base=True)
+        self.base_kind = base_kind or kind_from_path(self.path, base=True)
         self.abspath = self.path.expanduser().absolute()
 
+        if self.kind == "tasks":
+            self.parent = _guess_parent(self)
+
+        if self.kind == "yaml":
+            _ = self.data  # pylint: disable=pointless-statement
+
+    def _guess_kind(self) -> None:
+        if self.kind == "yaml":
+            if (
+                isinstance(self.data, list)
+                and len(self.data) > 0
+                and (
+                    "hosts" in self.data[0]
+                    or "import_playbook" in self.data[0]
+                    or "ansible.builtin.import_playbook" in self.data[0]
+                )
+            ):
+                if "rules" not in self.data[0]:
+                    self.kind = "playbook"
+                else:
+                    self.kind = "rulebook"
+            # we we failed to guess the more specific kind, we warn user
+            if self.kind == "yaml":
+                _logger.debug(
+                    "Passed '%s' positional argument was identified as generic '%s' file kind.",
+                    self.name,
+                    self.kind,
+                )
+
     def __getitem__(self, key: Any) -> Any:
         """Provide compatibility subscriptable support."""
         if key == "path":
             return str(self.path)
         if key == "type":
             return str(self.kind)
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def get(self, key: Any, default: Any = None) -> Any:
         """Provide compatibility subscriptable support."""
@@ -256,13 +292,8 @@ class Lintable:
 
     def _populate_content_cache_from_disk(self) -> None:
         # Can raise UnicodeDecodeError
-        try:
-            self._content = self.path.expanduser().resolve().read_text(encoding="utf-8")
-        except FileNotFoundError as ex:
-            if vars(options).get("progressive"):
-                self._content = ""
-            else:
-                raise ex
+        self._content = self.path.expanduser().resolve().read_text(encoding="utf-8")
+
         if self._original_content is None:
             self._original_content = self._content
 
@@ -281,7 +312,8 @@ class Lintable:
         has not already been populated.
         """
         if not isinstance(value, str):
-            raise TypeError(f"Expected str but got {type(value)}")
+            msg = f"Expected str but got {type(value)}"
+            raise TypeError(msg)
         if self._original_content is None:
             if self._content is not None:
                 self._original_content = self._content
@@ -298,7 +330,7 @@ class Lintable:
         """Reset the internal content cache."""
         self._content = None
 
-    def write(self, force: bool = False) -> None:
+    def write(self, *, force: bool = False) -> None:
         """Write the value of ``Lintable.content`` to disk.
 
         This only writes to disk if the content has been updated (``Lintable.updated``).
@@ -320,7 +352,8 @@ class Lintable:
             # No changes to write.
             return
         self.path.expanduser().resolve().write_text(
-            self._content or "", encoding="utf-8"
+            self._content or "",
+            encoding="utf-8",
         )
 
     def __hash__(self) -> int:
@@ -340,26 +373,49 @@ class Lintable:
     @property
     def data(self) -> Any:
         """Return loaded data representation for current file, if possible."""
-        if not self._data:
-            if str(self.base_kind) == "text/yaml":
-                from ansiblelint.utils import (  # pylint: disable=import-outside-toplevel
-                    parse_yaml_linenumbers,
-                )
+        if self.state == States.NOT_LOADED:
+            if self.path.is_dir():
+                self.state = None
+                return self.state
+            try:
+                if str(self.base_kind) == "text/yaml":
+                    from ansiblelint.utils import (  # pylint: disable=import-outside-toplevel
+                        parse_yaml_linenumbers,
+                    )
 
-                self._data = parse_yaml_linenumbers(self)
-                # Lazy import to avoid delays and cyclic-imports
-                if "append_skipped_rules" not in globals():
-                    # pylint: disable=import-outside-toplevel
-                    from ansiblelint.skip_utils import append_skipped_rules
+                    self.state = parse_yaml_linenumbers(self)
+                    # now that _data is not empty, we can try guessing if playbook or rulebook
+                    # it has to be done before append_skipped_rules() call as it's relying
+                    # on self.kind.
+                    if self.kind == "yaml":
+                        self._guess_kind()
+                    # Lazy import to avoid delays and cyclic-imports
+                    if "append_skipped_rules" not in globals():
+                        # pylint: disable=import-outside-toplevel
+                        from ansiblelint.skip_utils import append_skipped_rules
 
-                self._data = append_skipped_rules(self._data, self)
+                    self.state = append_skipped_rules(self.state, self)
+                else:
+                    logging.debug(
+                        "data set to None for %s due to being of %s kind.",
+                        self.path,
+                        self.base_kind or "unknown",
+                    )
+                    self.state = States.UNKNOWN_DATA
 
-            # will remain None if we do not know how to load that file-type
-        return self._data
+            except (
+                RuntimeError,
+                FileNotFoundError,
+                YAMLError,
+                UnicodeDecodeError,
+            ) as exc:
+                self.state = States.LOAD_FAILED
+                self.exc = exc
+        return self.state
 
 
 # pylint: disable=redefined-outer-name
-def discover_lintables(options: Namespace) -> dict[str, Any]:
+def discover_lintables(options: Options) -> dict[str, Any]:
     """Find all files that we know how to lint.
 
     Return format is normalized, relative for stuff below cwd, ~/ for content
@@ -367,50 +423,59 @@ def discover_lintables(options: Namespace) -> dict[str, Any]:
     """
     # git is preferred as it also considers .gitignore
     git_command_present = [
-        "git",
+        *GIT_CMD,
         "ls-files",
         "--cached",
         "--others",
         "--exclude-standard",
         "-z",
     ]
-    git_command_absent = ["git", "ls-files", "--deleted", "-z"]
+    git_command_absent = [*GIT_CMD, "ls-files", "--deleted", "-z"]
     out = None
 
     try:
         out_present = subprocess.check_output(
-            git_command_present, stderr=subprocess.STDOUT, text=True
+            git_command_present,  # noqa: S603
+            stderr=subprocess.STDOUT,
+            text=True,
         ).split("\x00")[:-1]
         _logger.info(
-            "Discovered files to lint using: %s", " ".join(git_command_present)
+            "Discovered files to lint using: %s",
+            " ".join(git_command_present),
         )
 
         out_absent = subprocess.check_output(
-            git_command_absent, stderr=subprocess.STDOUT, text=True
+            git_command_absent,  # noqa: S603
+            stderr=subprocess.STDOUT,
+            text=True,
         ).split("\x00")[:-1]
         _logger.info("Excluded removed files using: %s", " ".join(git_command_absent))
 
         out = set(out_present) - set(out_absent)
     except subprocess.CalledProcessError as exc:
         if not (exc.returncode == 128 and "fatal: not a git repository" in exc.output):
-            _logger.warning(
-                "Failed to discover lintable files using git: %s",
-                exc.output.rstrip("\n"),
-            )
+            err = exc.output.rstrip("\n")
+            warn_or_fail(f"Failed to discover lintable files using git: {err}")
     except FileNotFoundError as exc:
         if options.verbosity:
-            _logger.warning("Failed to locate command: %s", exc)
+            warn_or_fail(f"Failed to locate command: {exc}")
 
-    if out is None:
-        exclude_pattern = "|".join(str(x) for x in options.exclude_paths)
-        _logger.info("Looking up for files, excluding %s ...", exclude_pattern)
-        # remove './' prefix from output of WcMatch
-        out = {
-            strip_dotslash_prefix(fname)
-            for fname in WcMatch(
-                ".", exclude_pattern=exclude_pattern, flags=RECURSIVE, limit=256
-            ).match()
-        }
+    # Applying exclude patterns
+    if not out:
+        out = set(".")
+
+    exclude_pattern = "|".join(str(x) for x in options.exclude_paths)
+    _logger.info("Looking up for files, excluding %s ...", exclude_pattern)
+    # remove './' prefix from output of WcMatch
+    out = {
+        strip_dotslash_prefix(fname)
+        for fname in wcmatch.wcmatch.WcMatch(
+            ".",
+            exclude_pattern=exclude_pattern,
+            flags=wcmatch.wcmatch.RECURSIVE,
+            limit=256,
+        ).match()
+    }
 
     return OrderedDict.fromkeys(sorted(out))
 
@@ -420,49 +485,60 @@ def strip_dotslash_prefix(fname: str) -> str:
     return fname[2:] if fname.startswith("./") else fname
 
 
-def guess_project_dir(config_file: str | None) -> str:
-    """Return detected project dir or current working directory."""
-    path = None
-    if config_file is not None and config_file != "/dev/null":
-        target = pathlib.Path(config_file)
-        if target.exists():
-            # for config inside .config, we return the parent dir as project dir
-            cfg_path = target.parent
-            if cfg_path.parts[-1] == ".config":
-                path = str(cfg_path.parent.absolute())
-            else:
-                path = str(cfg_path.absolute())
+def find_project_root(
+    srcs: Sequence[str],
+    config_file: str | None = None,
+) -> tuple[Path, str]:
+    """Return a directory containing .git or ansible-lint config files.
 
-    if path is None:
-        try:
-            result = subprocess.run(
-                ["git", "rev-parse", "--show-toplevel"],
-                capture_output=True,
-                text=True,
-                check=True,
-            )
+    That directory will be a common parent of all files and directories
+    passed in `srcs`.
 
-            path = result.stdout.splitlines()[0]
-        except subprocess.CalledProcessError as exc:
-            if not (
-                exc.returncode == 128 and "fatal: not a git repository" in exc.stderr
-            ):
-                _logger.warning(
-                    "Failed to guess project directory using git: %s",
-                    exc.stderr.rstrip("\n"),
-                )
-        except FileNotFoundError as exc:
-            _logger.warning("Failed to locate command: %s", exc)
+    If no directory in the tree contains a marker that would specify it's the
+    project root, the root of the file system is returned.
 
-    if path is None:
-        path = os.getcwd()
+    Returns a two-tuple with the first element as the project root path and
+    the second element as a string describing the method by which the
+    project root was discovered.
+    """
+    directory = None
+    if not srcs:
+        srcs = [str(Path.cwd().resolve().absolute())]
+    path_srcs = [Path(Path.cwd(), src).resolve() for src in srcs]
 
-    _logger.info(
-        "Guessed %s as project root directory",
-        path,
+    cfg_files = [config_file] if config_file else CONFIG_FILENAMES
+
+    # A list of lists of parents for each 'src'. 'src' is included as a
+    # "parent" of itself if it is a directory
+    src_parents = [
+        list(path.parents) + ([path] if path.is_dir() else []) for path in path_srcs
+    ]
+
+    common_base = max(
+        set.intersection(*(set(parents) for parents in src_parents)),
+        key=lambda path: path.parts,
     )
 
-    return path
+    for directory in (common_base, *common_base.parents):
+        if (directory / ".git").exists():
+            return directory, ".git directory"
+
+        if (directory / ".hg").is_dir():
+            return directory, ".hg directory"
+
+        for cfg_file in cfg_files:
+            # note that if cfg_file is already absolute, 'directory' is ignored
+            resolved_cfg_path = directory / cfg_file
+            if resolved_cfg_path.is_file():
+                if os.path.isabs(cfg_file):
+                    directory = Path(cfg_file).parent
+                    if directory.name == ".config":
+                        directory = directory.parent
+                return directory, f"config file {resolved_cfg_path}"
+
+    if not directory:
+        return Path.cwd(), "current working directory"
+    return directory, "file system root"
 
 
 def expand_dirs_in_lintables(lintables: set[Lintable]) -> None:
@@ -483,3 +559,14 @@ def expand_dirs_in_lintables(lintables: set[Lintable]) -> None:
                 for filename in all_files:
                     if filename.startswith(str(item.path)):
                         lintables.add(Lintable(filename))
+
+
+def _guess_parent(lintable: Lintable) -> Lintable | None:
+    """Return a parent directory for a lintable."""
+    try:
+        if lintable.path.parents[2].name == "roles":
+            # role_name = lintable.parents[1].name
+            return Lintable(lintable.path.parents[1], kind="role")
+    except IndexError:
+        pass
+    return None
