@@ -5,6 +5,7 @@ import logging
 import re
 import sys
 from collections import namedtuple
+from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
 import black
@@ -23,6 +24,7 @@ from ansiblelint.yaml_utils import deannotate, nested_items_path
 
 if TYPE_CHECKING:
     from ansiblelint.errors import MatchError
+    from ansiblelint.utils import Task
 
 
 _logger = logging.getLogger(__package__)
@@ -31,7 +33,7 @@ KEYWORDS_WITH_IMPLICIT_TEMPLATE = ("changed_when", "failed_when", "until", "when
 Token = namedtuple("Token", "lineno token_type value")
 
 ignored_re = re.compile(
-    "|".join(
+    "|".join(  # noqa: FLY002
         [
             r"^Object of type method is not JSON serializable",
             r"^Unexpected templating type error occurred on",
@@ -45,7 +47,7 @@ ignored_re = re.compile(
             r"Unrecognized type <<class 'ansible.template.AnsibleUndefined'>> for (.*) filter <value>$",
             # https://github.com/ansible/ansible-lint/issues/3155
             r"^The '(.*)' test expects a dictionary$",
-        ]
+        ],
     ),
     flags=re.MULTILINE | re.DOTALL,
 )
@@ -59,7 +61,8 @@ class JinjaRule(AnsibleLintRule):
     tags = ["formatting"]
     version_added = "v6.5.0"
     _ansible_error_re = re.compile(
-        r"^(?P<error>.*): (?P<detail>.*)\. String: (?P<string>.*)$", flags=re.MULTILINE
+        r"^(?P<error>.*): (?P<detail>.*)\. String: (?P<string>.*)$",
+        flags=re.MULTILINE,
     )
 
     env = jinja2.Environment(trim_blocks=False)
@@ -74,7 +77,9 @@ class JinjaRule(AnsibleLintRule):
 
     # pylint: disable=too-many-branches,too-many-locals
     def matchtask(  # noqa: C901
-        self, task: dict[str, Any], file: Lintable | None = None
+        self,
+        task: Task,
+        file: Lintable | None = None,
     ) -> list[MatchError]:
         result = []
         try:
@@ -85,7 +90,7 @@ class JinjaRule(AnsibleLintRule):
                 if isinstance(v, str):
                     try:
                         template(
-                            basedir=file.dir if file else ".",
+                            basedir=file.path.parent if file else Path("."),
                             value=v,
                             variables=deannotate(task.get("vars", {})),
                             fail_on_error=True,  # we later decide which ones to ignore or not
@@ -98,13 +103,12 @@ class JinjaRule(AnsibleLintRule):
                         )
                         orig_exc_message = getattr(orig_exc, "message", str(orig_exc))
                         match = self._ansible_error_re.match(
-                            getattr(orig_exc, "message", str(orig_exc))
+                            getattr(orig_exc, "message", str(orig_exc)),
                         )
-                        if ignored_re.search(orig_exc_message):
-                            bypass = True
-                        elif isinstance(orig_exc, AnsibleParserError):
-                            # "An unhandled exception occurred while running the lookup plugin '...'. Error was a <class 'ansible.errors.AnsibleParserError'>, original message: Invalid filename: 'None'. Invalid filename: 'None'"
-
+                        if ignored_re.search(orig_exc_message) or isinstance(
+                            orig_exc,
+                            AnsibleParserError,
+                        ):
                             # An unhandled exception occurred while running the lookup plugin 'template'. Error was a <class 'ansible.errors.AnsibleError'>, original message: the template file ... could not be found for the lookup. the template file ... could not be found for the lookup
 
                             # ansible@devel (2.14) new behavior:
@@ -116,9 +120,8 @@ class JinjaRule(AnsibleLintRule):
                         ):
                             error = match.group("error")
                             detail = match.group("detail")
-                            # string = match.group("string")
                             if error.startswith(
-                                "template error while templating string"
+                                "template error while templating string",
                             ):
                                 bypass = False
                             elif detail.startswith("unable to locate collection"):
@@ -130,33 +133,36 @@ class JinjaRule(AnsibleLintRule):
                             # lookup plugin 'template' not found
                             bypass = True
 
-                        # AnsibleFilterError: 'obj must be a list of dicts or a nested dict'
                         # AnsibleError: template error while templating string: expected token ':', got '}'. String: {{ {{ '1' }} }}
                         # AnsibleError: template error while templating string: unable to locate collection ansible.netcommon. String: Foo {{ buildset_registry.host | ipwrap }}
                         if not bypass:
                             result.append(
                                 self.create_matcherror(
                                     message=str(exc),
-                                    linenumber=_get_error_line(task, path),
+                                    lineno=_get_error_line(task, path),
                                     filename=file,
                                     tag=f"{self.id}[invalid]",
-                                )
+                                ),
                             )
                             continue
                     reformatted, details, tag = self.check_whitespace(
-                        v, key=key, lintable=file
+                        v,
+                        key=key,
+                        lintable=file,
                     )
                     if reformatted != v:
                         result.append(
                             self.create_matcherror(
                                 message=self._msg(
-                                    tag=tag, value=v, reformatted=reformatted
+                                    tag=tag,
+                                    value=v,
+                                    reformatted=reformatted,
                                 ),
-                                linenumber=_get_error_line(task, path),
+                                lineno=_get_error_line(task, path),
                                 details=details,
                                 filename=file,
                                 tag=f"{self.id}[{tag}]",
-                            )
+                            ),
                         )
         except Exception as exc:
             _logger.info("Exception in JinjaRule.matchtask: %s", exc)
@@ -172,28 +178,35 @@ class JinjaRule(AnsibleLintRule):
         if str(file.kind) == "vars":
             data = parse_yaml_from_file(str(file.path))
             # pylint: disable=unused-variable
-            for key, v, path in nested_items_path(data):
+            for key, v, _path in nested_items_path(data):
                 if isinstance(v, AnsibleUnicode):
                     reformatted, details, tag = self.check_whitespace(
-                        v, key=key, lintable=file
+                        v,
+                        key=key,
+                        lintable=file,
                     )
                     if reformatted != v:
                         results.append(
                             self.create_matcherror(
                                 message=self._msg(
-                                    tag=tag, value=v, reformatted=reformatted
+                                    tag=tag,
+                                    value=v,
+                                    reformatted=reformatted,
                                 ),
-                                linenumber=v.ansible_pos[1],
+                                lineno=v.ansible_pos[1],
                                 details=details,
                                 filename=file,
                                 tag=f"{self.id}[{tag}]",
-                            )
+                            ),
                         )
             if raw_results:
                 lines = file.content.splitlines()
                 for match in raw_results:
-                    # linenumber starts with 1, not zero
-                    skip_list = get_rule_skips_from_line(lines[match.linenumber - 1])
+                    # lineno starts with 1, not zero
+                    skip_list = get_rule_skips_from_line(
+                        line=lines[match.lineno - 1],
+                        lintable=file,
+                    )
                     if match.rule.id not in skip_list and match.tag not in skip_list:
                         results.append(match)
         else:
@@ -215,7 +228,7 @@ class JinjaRule(AnsibleLintRule):
         new_text = self.unlex(tokens)
         if text != new_text:
             _logger.debug(
-                "Unable to perform full roundtrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}"
+                "Unable to perform full roundtrip lex-unlex on jinja template (expected when '-' modifier is used): {text} -> {new_text}",
             )
         return tokens
 
@@ -234,7 +247,10 @@ class JinjaRule(AnsibleLintRule):
 
     # pylint: disable=too-many-branches,too-many-statements,too-many-locals
     def check_whitespace(  # noqa: max-complexity: 13
-        self, text: str, key: str, lintable: Lintable | None = None
+        self,
+        text: str,
+        key: str,
+        lintable: Lintable | None = None,
     ) -> tuple[str, str, str]:
         """Check spacing inside given jinja2 template string.
 
@@ -244,7 +260,7 @@ class JinjaRule(AnsibleLintRule):
         :returns: (string, string, string)  reformatted text, detailed error, error tag
         """
 
-        def cook(value: str, implicit: bool = False) -> str:
+        def cook(value: str, *, implicit: bool = False) -> str:
             """Prepare an implicit string for jinja parsing when needed."""
             if not implicit:
                 return value
@@ -253,7 +269,7 @@ class JinjaRule(AnsibleLintRule):
                 return value
             return f"{{{{ {value} }}}}"
 
-        def uncook(value: str, implicit: bool = False) -> str:
+        def uncook(value: str, *, implicit: bool = False) -> str:
             """Restore an string to original form when it was an implicit one."""
             if not implicit:
                 return value
@@ -307,7 +323,7 @@ class JinjaRule(AnsibleLintRule):
                     # process expression
                     # pylint: disable=unsupported-membership-test
                     if isinstance(expr_str, str) and "\n" in expr_str:
-                        raise NotImplementedError()
+                        raise NotImplementedError
                     leading_spaces = " " * (len(expr_str) - len(expr_str.lstrip()))
                     expr_str = leading_spaces + blacken(expr_str.lstrip())
                     if tokens[
@@ -337,12 +353,12 @@ class JinjaRule(AnsibleLintRule):
             # newlines, as we decided to not touch them yet.
             # These both are documented as known limitations.
             _logger.debug("Ignored jinja internal error %s", exc)
-            return uncook(text, implicit), "", "spacing"
+            return uncook(text, implicit=implicit), "", "spacing"
 
         # finalize
         reformatted = self.unlex(tokens)
         failed = reformatted != text
-        reformatted = uncook(reformatted, implicit)
+        reformatted = uncook(reformatted, implicit=implicit)
         details = (
             f"Jinja2 template rewrite recommendation: `{reformatted}`."
             if failed
@@ -354,11 +370,12 @@ class JinjaRule(AnsibleLintRule):
 def blacken(text: str) -> str:
     """Format Jinja2 template using black."""
     return black.format_str(
-        text, mode=black.FileMode(line_length=sys.maxsize, string_normalization=False)
+        text,
+        mode=black.FileMode(line_length=sys.maxsize, string_normalization=False),
     ).rstrip("\n")
 
 
-if "pytest" in sys.modules:  # noqa: C901
+if "pytest" in sys.modules:
     import pytest
 
     from ansiblelint.rules import RulesCollection  # pylint: disable=ungrouped-imports
@@ -377,15 +394,16 @@ if "pytest" in sys.modules:  # noqa: C901
         collection.register(JinjaRule())
         lintable = Lintable("examples/playbooks/jinja-spacing.yml")
         results = Runner(lintable, rules=collection).run()
-        return list(map(lambda item: item.linenumber, results))
+        return [item.lineno for item in results]
 
     def test_jinja_spacing_playbook(
-        error_expected_lines: list[int], lint_error_lines: list[int]
+        error_expected_lines: list[int],
+        lint_error_lines: list[int],
     ) -> None:
         """Ensure that expected error lines are matching found linting error lines."""
         # list unexpected error lines or non-matching error lines
         error_lines_difference = list(
-            set(error_expected_lines).symmetric_difference(set(lint_error_lines))
+            set(error_expected_lines).symmetric_difference(set(lint_error_lines)),
         )
         assert len(error_lines_difference) == 0
 
@@ -399,7 +417,7 @@ if "pytest" in sys.modules:  # noqa: C901
         error_expected_lineno = [14, 15, 16, 17, 18, 19, 32]
         assert len(results) == len(error_expected_lineno)
         for idx, err in enumerate(results):
-            assert err.linenumber == error_expected_lineno[idx]
+            assert err.lineno == error_expected_lineno[idx]
 
     @pytest.mark.parametrize(
         ("text", "expected", "tag"),
@@ -422,7 +440,10 @@ if "pytest" in sys.modules:  # noqa: C901
                 id="5",
             ),
             pytest.param(
-                "Shell with jinja filter", "Shell with jinja filter", "spacing", id="6"
+                "Shell with jinja filter",
+                "Shell with jinja filter",
+                "spacing",
+                id="6",
             ),
             pytest.param(
                 "{{{'dummy_2':1}|true}}",
@@ -446,10 +467,7 @@ if "pytest" in sys.modules:  # noqa: C901
             pytest.param("{{foo(123)}}", "{{ foo(123) }}", "spacing", id="11"),
             pytest.param("{{ foo(a.b.c) }}", "{{ foo(a.b.c) }}", "spacing", id="12"),
             # pytest.param(
-            #     "{{ foo | bool else [ ] }}",
-            #     "{{ foo | bool else [] }}",
             #     "spacing",
-            #     id="13",
             # ),
             pytest.param(
                 "{{foo(x =['server_options'])}}",
@@ -458,7 +476,10 @@ if "pytest" in sys.modules:  # noqa: C901
                 id="14",
             ),
             pytest.param(
-                '{{ [ "host", "NA"] }}', '{{ ["host", "NA"] }}', "spacing", id="15"
+                '{{ [ "host", "NA"] }}',
+                '{{ ["host", "NA"] }}',
+                "spacing",
+                id="15",
             ),
             pytest.param(
                 "{{ {'dummy_2': {'nested_dummy_1': value_1,\n    'nested_dummy_2': value_2}} |\ncombine(dummy_1) }}",
@@ -511,7 +532,10 @@ if "pytest" in sys.modules:  # noqa: C901
             ),
             pytest.param("{{ r(1,[]) }}", "{{ r(1, []) }}", "spacing", id="26"),
             pytest.param(
-                "{{ lookup([ddd ]) }}", "{{ lookup([ddd]) }}", "spacing", id="27"
+                "{{ lookup([ddd ]) }}",
+                "{{ lookup([ddd]) }}",
+                "spacing",
+                id="27",
             ),
             pytest.param(
                 "{{ [ x ] if x is string else x }}",
@@ -520,7 +544,6 @@ if "pytest" in sys.modules:  # noqa: C901
                 id="28",
             ),
             pytest.param(
-                # "{% if a|int <= 8 -%} iptables {%- else -%} iptables-nft {%- endif %}",
                 "{% if a|int <= 8 -%} iptables {%- else -%} iptables-nft {%- endif %}",
                 "{% if a | int <= 8 -%} iptables{%- else -%} iptables-nft{%- endif %}",
                 "spacing",
@@ -570,7 +593,10 @@ if "pytest" in sys.modules:  # noqa: C901
             ),
             pytest.param("{{ a +~'b' }}", "{{ a + ~'b' }}", "spacing", id="36"),
             pytest.param(
-                "{{ (a[: -4] *~ b) }}", "{{ (a[:-4] * ~b) }}", "spacing", id="37"
+                "{{ (a[: -4] *~ b) }}",
+                "{{ (a[:-4] * ~b) }}",
+                "spacing",
+                id="37",
             ),
             pytest.param("{{ [a,~ b] }}", "{{ [a, ~b] }}", "spacing", id="38"),
             # Not supported yet due to being accepted by black:
@@ -618,7 +644,9 @@ if "pytest" in sys.modules:  # noqa: C901
         rule = JinjaRule()
 
         reformatted, details, returned_tag = rule.check_whitespace(
-            text, key="name", lintable=Lintable("playbook.yml")
+            text,
+            key="name",
+            lintable=Lintable("playbook.yml"),
         )
         assert tag == returned_tag, details
         assert expected == reformatted
@@ -649,7 +677,9 @@ if "pytest" in sys.modules:  # noqa: C901
         # implicit jinja2 are working only inside playbooks and tasks
         lintable = Lintable(name="playbook.yml", kind="playbook")
         reformatted, details, returned_tag = rule.check_whitespace(
-            text, key="when", lintable=lintable
+            text,
+            key="when",
+            lintable=lintable,
         )
         assert tag == returned_tag, details
         assert expected == reformatted
@@ -678,10 +708,10 @@ if "pytest" in sys.modules:  # noqa: C901
         assert len(errs) == 2
         assert errs[0].tag == "jinja[spacing]"
         assert errs[0].rule.id == "jinja"
-        assert errs[0].linenumber == 9
+        assert errs[0].lineno == 9
         assert errs[1].tag == "jinja[invalid]"
         assert errs[1].rule.id == "jinja"
-        assert errs[1].linenumber == 9
+        assert errs[1].lineno == 9
 
     def test_jinja_valid() -> None:
         """Tests our ability to parse jinja, even when variables may not be defined."""
@@ -701,5 +731,6 @@ def _get_error_line(task: dict[str, Any], path: list[str | int]) -> int:
         if LINE_NUMBER_KEY in ctx:
             line = ctx[LINE_NUMBER_KEY]
     if not isinstance(line, int):
-        raise RuntimeError("Line number is not an integer")
+        msg = "Line number is not an integer"
+        raise RuntimeError(msg)
     return line
