@@ -2,7 +2,7 @@
 import pytest
 
 from ansiblelint._mockings import _make_module_stub
-from ansiblelint.constants import INVALID_CONFIG_RC
+from ansiblelint.constants import RC
 
 
 def test_make_module_stub() -> None:
@@ -10,4 +10,4 @@ def test_make_module_stub() -> None:
     with pytest.raises(SystemExit) as exc:
         _make_module_stub("")
     assert exc.type == SystemExit
-    assert exc.value.code == INVALID_CONFIG_RC
+    assert exc.value.code == RC.INVALID_CONFIG
