@@ -18,42 +18,52 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
-import os
-from typing import Any, List, Set, Type
+from __future__ import annotations
+
+from pathlib import Path
+from typing import TYPE_CHECKING, Any
 
 import pytest
 
 from ansiblelint import formatters
-from ansiblelint.file_utils import Lintable, abspath
-from ansiblelint.rules import RulesCollection
+from ansiblelint.file_utils import Lintable
 from ansiblelint.runner import Runner
 
-LOTS_OF_WARNINGS_PLAYBOOK = abspath(
-    "examples/playbooks/lots_of_warnings.yml", os.getcwd()
-)
+if TYPE_CHECKING:
+    from ansiblelint.rules import RulesCollection
+
+LOTS_OF_WARNINGS_PLAYBOOK = Path("examples/playbooks/lots_of_warnings.yml").resolve()
 
 
 @pytest.mark.parametrize(
     ("playbook", "exclude", "length"),
     (
-        pytest.param("examples/playbooks/nomatchestest.yml", [], 0, id="nomatchestest"),
-        pytest.param("examples/playbooks/unicode.yml", [], 1, id="unicode"),
+        pytest.param(
+            Path("examples/playbooks/nomatchestest.yml"),
+            [],
+            0,
+            id="nomatchestest",
+        ),
+        pytest.param(Path("examples/playbooks/unicode.yml"), [], 1, id="unicode"),
         pytest.param(
             LOTS_OF_WARNINGS_PLAYBOOK,
             [LOTS_OF_WARNINGS_PLAYBOOK],
             0,
             id="lots_of_warnings",
         ),
-        pytest.param("examples/playbooks/become.yml", [], 0, id="become"),
+        pytest.param(Path("examples/playbooks/become.yml"), [], 0, id="become"),
         pytest.param(
-            "examples/playbooks/contains_secrets.yml", [], 0, id="contains_secrets"
+            Path("examples/playbooks/contains_secrets.yml"),
+            [],
+            0,
+            id="contains_secrets",
         ),
     ),
 )
 def test_runner(
     default_rules_collection: RulesCollection,
-    playbook: str,
-    exclude: List[str],
+    playbook: Path,
+    exclude: list[str],
     length: int,
 ) -> None:
     """Test that runner can go through any corner cases."""
@@ -78,7 +88,8 @@ def test_runner_exclude_paths(default_rules_collection: RulesCollection) -> None
 
 @pytest.mark.parametrize(("exclude_path"), ("**/playbooks/*.yml",))
 def test_runner_exclude_globs(
-    default_rules_collection: RulesCollection, exclude_path: str
+    default_rules_collection: RulesCollection,
+    exclude_path: str,
 ) -> None:
     """Test that globs work."""
     runner = Runner(
@@ -88,8 +99,8 @@ def test_runner_exclude_globs(
     )
 
     matches = runner.run()
-    # we expect to find one error from the only .yaml file we have there.
-    assert len(matches) == 1
+    # we expect to find one 2 matches from the very few .yaml file we have there (most of them have .yml extension)
+    assert len(matches) == 2
 
 
 @pytest.mark.parametrize(
@@ -103,10 +114,10 @@ def test_runner_exclude_globs(
 )
 def test_runner_unicode_format(
     default_rules_collection: RulesCollection,
-    formatter_cls: Type[formatters.BaseFormatter[Any]],
+    formatter_cls: type[formatters.BaseFormatter[Any]],
 ) -> None:
     """Check that all formatters are unicode-friendly."""
-    formatter = formatter_cls(os.getcwd(), display_relative_path=True)
+    formatter = formatter_cls(Path.cwd(), display_relative_path=True)
     runner = Runner(
         Lintable("examples/playbooks/unicode.yml", kind="playbook"),
         rules=default_rules_collection,
@@ -114,12 +125,19 @@ def test_runner_unicode_format(
 
     matches = runner.run()
 
-    formatter.format(matches[0])
+    formatter.apply(matches[0])
 
 
-@pytest.mark.parametrize("directory_name", ("test/", os.path.abspath("test")))
+@pytest.mark.parametrize(
+    "directory_name",
+    (
+        pytest.param(Path("test/fixtures/verbosity-tests"), id="rel"),
+        pytest.param(Path("test/fixtures/verbosity-tests").resolve(), id="abs"),
+    ),
+)
 def test_runner_with_directory(
-    default_rules_collection: RulesCollection, directory_name: str
+    default_rules_collection: RulesCollection,
+    directory_name: Path,
 ) -> None:
     """Check that runner detects a directory as role."""
     runner = Runner(directory_name, rules=default_rules_collection)
@@ -130,9 +148,9 @@ def test_runner_with_directory(
 
 def test_files_not_scanned_twice(default_rules_collection: RulesCollection) -> None:
     """Ensure that lintables aren't double-checked."""
-    checked_files: Set[Lintable] = set()
+    checked_files: set[Lintable] = set()
 
-    filename = os.path.abspath("examples/playbooks/common-include-1.yml")
+    filename = Path("examples/playbooks/common-include-1.yml").resolve()
     runner = Runner(
         filename,
         rules=default_rules_collection,
@@ -143,9 +161,9 @@ def test_files_not_scanned_twice(default_rules_collection: RulesCollection) -> N
     assert len(runner.checked_files) == 2
     assert len(run1) == 1
 
-    filename = os.path.abspath("examples/playbooks/common-include-2.yml")
+    filename = Path("examples/playbooks/common-include-2.yml").resolve()
     runner = Runner(
-        filename,
+        str(filename),
         rules=default_rules_collection,
         verbosity=0,
         checked_files=checked_files,
